/**********************************************/
/* mqqmobj.hpp ... Queue Mgr obj declarations.*/
/* Part of SoftWoehr Library for IBM MQSeries */
/* Copyright *C* 1999 Jack J. Woehr           */
/* PO Box 51, Golden, Colorado 80402-0051     */
/* http://www.well.com/user/jax/SoftWoehr     */
/* jwoehr@ibm.net jax@well.com                */
/* ********************************************/
/* This is open source software. Please see   */
/* file license.txt. THERE IS NO WARRANTY.    */
/**********************************************/
#ifndef SW_MQQMOBJ_HPP
#define SW_MQQMOBJ_HPP

#ifndef SW_MQOBJ_HPP
   #include "mqobj.hpp"
#endif // SW_MQOBJ_HPP

#ifndef SW_MQINQ_HPP
   #include "mqinq.hpp"
#endif // SW_MQINQ_HPP

namespace SoftWoehr {

/** Represents a queue manager as an object */
class MQQueueManagerObject : public MQObject
   {
   public   :

      /** Create an MQQueueManagerObject with the descriptor already set to Queue */
      MQQueueManagerObject();

      /** Dtor */
      virtual ~MQQueueManagerObject();

      /** Open this object in style of superclass.
        * This overload serves merely to keep the very picky Sun
        * Sun CC 5.0 * compiler from complaining about name hiding.
        */
      virtual MQObject & open ( MQQueueManager & queue_manager
                              , const Descriptor& descriptor
                              , const Options   & options
                              )
        {
        return MQObject::open( queue_manager
                             , descriptor
                             , options
                             )
                             ;
        }

      /** Open a given queue manager as an object. */
      virtual MQQueueManagerObject &
         open( MQQueueManager & queue_manager
             , const MQObject::Options & options
             )
             ;

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* The code within these blocks was autogenerated    */
/* using the following files found in $JXMQ/pl.      */
/* atrqmihp.pl atrqmicp.pl atrqmchp.pl atrqmccp.pl   */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/** Character attribute factory for MQCA_COMMAND_INPUT_Q_NAME */
static MQInquiry::character_attribute *
   mqca_command_input_q_name(void);

/** Character attribute factory for MQCA_DEAD_LETTER_Q_NAME */
static MQInquiry::character_attribute *
   mqca_dead_letter_q_name(void);

/** Character attribute factory for MQCA_DEF_XMIT_Q_NAME */
static MQInquiry::character_attribute *
   mqca_def_xmit_q_name(void);

/** Character attribute factory for MQCA_Q_MGR_DESC */
static MQInquiry::character_attribute *
   mqca_q_mgr_desc(void);

/** Character attribute factory for MQCA_Q_MGR_NAME */
static MQInquiry::character_attribute *
   mqca_q_mgr_name(void);

/** Integer attribute factory for MQIA_CODED_CHAR_SET_ID */
static MQInquiry::integer_attribute *
   mqia_coded_char_set_id(void);

/** Integer attribute factory for MQIA_COMMAND_LEVEL */
static MQInquiry::integer_attribute *
   mqia_command_level(void);

/** Integer attribute factory for MQIA_MAX_HANDLES */
static MQInquiry::integer_attribute *
   mqia_max_handles(void);

/** Integer attribute factory for MQIA_MAX_MSG_LENGTH */
static MQInquiry::integer_attribute *
   mqia_max_msg_length(void);

/** Integer attribute factory for MQIA_MAX_PRIORITY */
static MQInquiry::integer_attribute *
   mqia_max_priority(void);

/** Integer attribute factory for MQIA_MAX_UNCOMMITTED_MSGS */
static MQInquiry::integer_attribute *
   mqia_max_uncommitted_msgs(void);

/** Integer attribute factory for MQIA_PLATFORM */
static MQInquiry::integer_attribute *
   mqia_platform(void);

/** Integer attribute factory for MQIA_SYNCPOINT */
static MQInquiry::integer_attribute *
   mqia_syncpoint(void);

/** Integer attribute factory for MQIA_TRIGGER_INTERVAL */
static MQInquiry::integer_attribute *
   mqia_trigger_interval(void);

/** Integer attribute factory for MQIA_AUTHORITY_EVENT */
static MQInquiry::integer_attribute *
   mqia_authority_event(void);

/** Integer attribute factory for MQIA_INHIBIT_EVENT */
static MQInquiry::integer_attribute *
   mqia_inhibit_event(void);

/** Integer attribute factory for MQIA_LOCAL_EVENT */
static MQInquiry::integer_attribute *
   mqia_local_event(void);

/** Integer attribute factory for MQIA_REMOTE_EVENT */
static MQInquiry::integer_attribute *
   mqia_remote_event(void);

/** Integer attribute factory for MQIA_START_STOP_EVENT */
static MQInquiry::integer_attribute *
   mqia_start_stop_event(void);

/** Integer attribute factory for MQIA_PERFORMANCE_EVENT */
static MQInquiry::integer_attribute *
   mqia_performance_event(void);

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* End of autogenerated code.                        */
/* using the following files found in $JXMQ/pl.      */
/* atrqmihp.pl atrqmicp.pl atrqmchp.pl atrqmccp.pl   */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/** Compose an inquiry which applies to any queue mgr.
  * Ask everything you can MQINQ about a queue.
  */
static MQInquiry * inquiry_any_queue_manager(void);

};                          /* class MQQueueManagerObject : public MQObject*/

} // namespace SoftWoehr {

#endif                                 /* SW_MQQMOBJ_HPP            */
